/****************************************************************************************** 
 *	Chili DirectX Framework Version 11.12.17											  *	
 *	D3DGraphics.cpp																		  *
 *	Copyright 2011 PlanetChili.net														  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "D3DGraphics.h"
#include <math.h>

D3DGraphics::D3DGraphics( HWND hWnd )
{
	pDirect3D = Direct3DCreate9( D3D_SDK_VERSION );

    D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory( &d3dpp,sizeof( d3dpp ) );
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_ONE;
	d3dpp.Flags = D3DPRESENTFLAG_LOCKABLE_BACKBUFFER;

    pDirect3D->CreateDevice( D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,hWnd,
		D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DCREATE_PUREDEVICE,&d3dpp,&pDevice );
}

D3DGraphics::~D3DGraphics()
{
	if( pDevice )
	{
		pDevice->Release();
		pDevice = NULL;
	}
	if( pDirect3D )
	{
		pDirect3D->Release();
		pDirect3D = NULL;
	}
}

void D3DGraphics::PutPixel( int x,int y,int r,int g,int b )
{
	IDirect3DSurface9* pBackBuffer = NULL;
	D3DLOCKED_RECT rect;

	pDevice->GetBackBuffer( 0,0,D3DBACKBUFFER_TYPE_MONO,&pBackBuffer );
	pBackBuffer->LockRect( &rect,NULL,NULL );
	((D3DCOLOR*)rect.pBits)[ x + (rect.Pitch >> 2) * y ] = D3DCOLOR_XRGB( r,g,b );
	pBackBuffer->UnlockRect();
	pBackBuffer->Release();
}

void D3DGraphics::BeginFrame()
{
	pDevice->Clear( 0,NULL,D3DCLEAR_TARGET,D3DCOLOR_XRGB(0,0,0),0.0f,0 );
}

void D3DGraphics::EndFrame()
{
	pDevice->Present( NULL,NULL,NULL,NULL );
}
void D3DGraphics::DrawLine(int x1,int y1,int x2,int y2,int r, int g,int bl){
	int dx = x2-x1;
	int dy = y2-y1;
	
	if( abs(dy)>abs(dx)){ // calculates the x values relative to y - its an mirror function of the y = f(x)
		if(y1>y2){
			int temp = y2;//swaps the values of y1 and y2 so that drawing loop gets executed for this condition(see video - Beginner Lesson 10)
			y2=y1;
			y1=temp;
		temp = x2;//swaps the values of x2 and x1
		x2=x1;
		x1=temp;
		}

		float m = (float)dx/(float)dy; // calculates the steepness of the line
		float b = x1-m*y1;					// calculates the y-intercept
		for(int y = y1; y<=y2; y++){
			int x = m*y+b + 0.5f; // 0.5f corrects the rounding effect of compiler(see video)
			PutPixel(x,y,r,g,bl);
		}
	}
	else{
		if(x1>x2){
			int temp = y2;//swaps the values of y1 and y2 so that drawing loop gets executed for this condition(see video)
			y2=y1;
			y1=temp;
		temp = x2;//swaps the values of x2 and x1
		x2=x1;
		x1=temp;
		}

		float m = (float)dy/(float)dx; // calculates the steepness of the line
		float b = y1-m*x1;					// calculates the y-intercept
		for(int x = x1; x<=x2; x++){
			int y = m*x+b + 0.5f;
			PutPixel(x,y,r,g,bl);
		}
	}
}