/****************************************************************************************** 
 *	Chili DirectX Framework Version 11.12.17											  *	
 *	Game.cpp																			  *
 *	Copyright 2011 PlanetChili.net														  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "Game.h"

Game::Game( HWND hWnd,const KeyboardServer& kServer )
:	gfx ( hWnd ),
	kbd( kServer ),
	size(512),//initializes width and height of square
	radius(32)//initializes radius of star and width/2 of the square
{}
void Game::Go()
{
	gfx.BeginFrame();
	ComposeFrame();
	gfx.EndFrame();
}

void Game::DrawCell(int cellx,int celly,int sq_w, int line_g, int r, int g, int b)
{
	/*  cellx, celly - center coordinates of the star(see drawing)
	sq_w  -  star radius
	line_g -line gap(see drawing) */

	int movex,movey;

	 movey=celly; 
	 for (movex=cellx-sq_w; movex <= cellx; movex=movex+line_g) //3rd quadrant
      {
         gfx.DrawLine(cellx-sq_w,movey,movex,celly+sq_w,r,g,b); // x1,y1(start point of the line) x2,y2(end point of the line),red,green,blue from the draw-line function
         movey=movey+line_g;
      }
	   
     movey=celly; 
   for (movex=cellx+sq_w; movex >= cellx; movex=movex-line_g)
      {
         gfx.DrawLine(cellx+sq_w,movey,movex,celly+sq_w,r,g,b); //4th quadrant
         movey=movey+line_g;
      }

     movey=celly; 
   for (movex=cellx+sq_w; movex >= cellx; movex=movex-line_g) //1st quadrant
      {
         gfx.DrawLine(cellx+sq_w,movey,movex,celly-sq_w,r,g,b); 
          movey=movey-line_g;
      }  

   movey=celly; 
   for (movex=cellx-sq_w; movex <= cellx; movex=movex+line_g)
      {
         gfx.DrawLine(cellx-sq_w,movey,movex,celly-sq_w,r,g,b); //2nd quadrant
         movey=movey-line_g;
      }  
}

void Game::DrawStar(int starx,int stary,int star_r, int line_g, int r, int g, int b)
{
	/*  starx, stary - center coordinates of the star(see drawing)
	star_r  -  star radius
	line_g -line gap(see drawing) */

	int movex,movey;
	 movey=stary-star_r; //top edge y-coordinate = star center - star radius
	 for (movex=starx; movex <= starx+star_r; movex=movex+line_g) //1st quadrant
      {
         gfx.DrawLine(starx,movey,movex,stary,r,g,b); // x1,y1(start point of the line) x2,y2(end point of the line),red,green,blue from the draw-line function
         movey=movey+line_g;
      }
	    movey=stary-star_r; //top edge y-coordinate = star center - star radius
   for (movex=starx; movex >= starx-star_r; movex=movex-line_g)
      {
         gfx.DrawLine(starx,movey,movex,stary,r,g,b); //2nd quadrant
         movey=movey+line_g;
      }

     movey=stary+star_r; //bottom edge y-coordinate = star center + star radius
   for (movex=starx; movex >= starx-star_r; movex=movex-line_g) //3rd quadrant
      {
         gfx.DrawLine(starx,movey,movex,stary,r,g,b); 
          movey=movey-line_g;
      }  

   movey=stary+star_r; //bottom edge y-coordinate = star center + star radius
   for (movex=starx; movex <= starx+star_r; movex=movex+line_g)
      {
         gfx.DrawLine(starx,movey,movex,stary,r,g,b); //4th quadrant
         movey=movey-line_g;
      }  
    
}


void Game::ComposeFrame()
{
	int x=100,y=50; //upper left corner coordinates 

	if(kbd.UpIsPressed()){//resize star radius or cell width
		radius=2*radius;
	}
	if(kbd.DownIsPressed()){
	radius = radius/2;
	}

int loopy = y+radius; // initializes y-axis variable and sets it back to upper side of the square when both loops end


   while( loopy < y + size )// comparison with the bottom side of the square(see the drawing)
   {
      int loopx = x+radius; // initializes x-axis variable and sets it back to left side of the square when inner loop ends
      
	  while( loopx < x + size )
      {
        DrawCell(loopx,loopy,radius,4,0,255,0);
		//DrawStar(loopx,loopy,radius,4,0,255,0);
         loopx=loopx+2*radius;
      }
      loopy=loopy+2*radius;
   }

}